@@ -33,6 +33,7 @@ pub struct AppSinkCallbacks {
             Box<dyn FnMut(&AppSink) -> Result<gst::FlowSuccess, gst::FlowError> + Send + 'static>,
         >,
     >,
+    new_event: Option<RefCell<Box<dyn FnMut(&AppSink) -> bool + Send + 'static>>>,
     panicked: AtomicBool,
     callbacks: ffi::GstAppSinkCallbacks,
 }
@@ -47,11 +48,13 @@ impl AppSinkCallbacks {
             eos: None,
             new_preroll: None,
             new_sample: None,
+            new_event: None,
         }
     }
 }
 
 #[allow(clippy::type_complexity)]
+#[must_use = "The builder must be built to be used"]
 pub struct AppSinkCallbacksBuilder {
     eos: Option<RefCell<Box<dyn FnMut(&AppSink) + Send + 'static>>>,
     new_preroll: Option<
@@ -64,6 +67,7 @@ pub struct AppSinkCallbacksBuilder {
             Box<dyn FnMut(&AppSink) -> Result<gst::FlowSuccess, gst::FlowError> + Send + 'static>,
         >,
     >,
+    new_event: Option<RefCell<Box<dyn FnMut(&AppSink) -> bool + Send + 'static>>>,
 }
 
 impl AppSinkCallbacksBuilder {
@@ -98,15 +102,27 @@ impl AppSinkCallbacksBuilder {
         }
     }
 
+    #[cfg(any(feature = "v1_20", feature = "dox"))]
+    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    pub fn new_event<F: FnMut(&AppSink) -> bool + Send + 'static>(self, new_event: F) -> Self {
+        Self {
+            new_event: Some(RefCell::new(Box::new(new_event))),
+            ..self
+        }
+    }
+
+    #[must_use = "Building the callbacks without using them has no effect"]
     pub fn build(self) -> AppSinkCallbacks {
         let have_eos = self.eos.is_some();
         let have_new_preroll = self.new_preroll.is_some();
         let have_new_sample = self.new_sample.is_some();
+        let have_new_event = self.new_event.is_some();
 
         AppSinkCallbacks {
             eos: self.eos,
             new_preroll: self.new_preroll,
             new_sample: self.new_sample,
+            new_event: self.new_event,
             panicked: AtomicBool::new(false),
             callbacks: ffi::GstAppSinkCallbacks {
                 eos: if have_eos { Some(trampoline_eos) } else { None },
@@ -120,12 +136,12 @@ impl AppSinkCallbacksBuilder {
                 } else {
                     None
                 },
-                _gst_reserved: [
-                    ptr::null_mut(),
-                    ptr::null_mut(),
-                    ptr::null_mut(),
-                    ptr::null_mut(),
-                ],
+                new_event: if have_new_event {
+                    Some(trampoline_new_event)
+                } else {
+                    None
+                },
+                _gst_reserved: [ptr::null_mut(), ptr::null_mut(), ptr::null_mut()],
             },
         }
     }
@@ -134,11 +150,11 @@ impl AppSinkCallbacksBuilder {
 fn post_panic_error_message(element: &AppSink, err: &dyn std::any::Any) {
     skip_assert_initialized!();
     if let Some(cause) = err.downcast_ref::<&str>() {
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked: {}", cause]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked: {}", cause]);
     } else if let Some(cause) = err.downcast_ref::<String>() {
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked: {}", cause]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked: {}", cause]);
     } else {
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked"]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked"]);
     }
 }
 
@@ -148,7 +164,7 @@ unsafe extern "C" fn trampoline_eos(appsink: *mut ffi::GstAppSink, callbacks: gp
 
     if callbacks.panicked.load(Ordering::Relaxed) {
         let element: Borrowed<AppSink> = from_glib_borrow(appsink);
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked"]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked"]);
         return;
     }
 
@@ -175,7 +191,7 @@ unsafe extern "C" fn trampoline_new_preroll(
 
     if callbacks.panicked.load(Ordering::Relaxed) {
         let element: Borrowed<AppSink> = from_glib_borrow(appsink);
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked"]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked"]);
         return gst::FlowReturn::Error.into_glib();
     }
 
@@ -208,7 +224,7 @@ unsafe extern "C" fn trampoline_new_sample(
 
     if callbacks.panicked.load(Ordering::Relaxed) {
         let element: Borrowed<AppSink> = from_glib_borrow(appsink);
-        gst::element_error!(&element, gst::LibraryError::Failed, ["Panicked"]);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked"]);
         return gst::FlowReturn::Error.into_glib();
     }
 
@@ -232,6 +248,39 @@ unsafe extern "C" fn trampoline_new_sample(
     ret.into_glib()
 }
 
+unsafe extern "C" fn trampoline_new_event(
+    appsink: *mut ffi::GstAppSink,
+    callbacks: gpointer,
+) -> glib::ffi::gboolean {
+    let callbacks = &*(callbacks as *const AppSinkCallbacks);
+    let element: Borrowed<AppSink> = from_glib_borrow(appsink);
+
+    if callbacks.panicked.load(Ordering::Relaxed) {
+        let element: Borrowed<AppSink> = from_glib_borrow(appsink);
+        gst::element_error!(element, gst::LibraryError::Failed, ["Panicked"]);
+        return false.into_glib();
+    }
+
+    let ret = if let Some(ref new_event) = callbacks.new_event {
+        let result = panic::catch_unwind(panic::AssertUnwindSafe(|| {
+            (&mut *new_event.borrow_mut())(&element)
+        }));
+        match result {
+            Ok(result) => result,
+            Err(err) => {
+                callbacks.panicked.store(true, Ordering::Relaxed);
+                post_panic_error_message(&element, &err);
+
+                false
+            }
+        }
+    } else {
+        false
+    };
+
+    ret.into_glib()
+}
+
 unsafe extern "C" fn destroy_callbacks(ptr: gpointer) {
     Box::<AppSinkCallbacks>::from_raw(ptr as *mut _);
 }
@@ -241,7 +290,7 @@ impl AppSink {
     pub fn set_callbacks(&self, callbacks: AppSinkCallbacks) {
         use once_cell::sync::Lazy;
         static SET_ONCE_QUARK: Lazy<glib::Quark> =
-            Lazy::new(|| glib::Quark::from_string("gstreamer-rs-app-sink-callbacks"));
+            Lazy::new(|| glib::Quark::from_str("gstreamer-rs-app-sink-callbacks"));
 
         unsafe {
             let sink = self.to_glib_none().0;
@@ -533,7 +582,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -561,7 +610,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -589,7 +638,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -617,7 +666,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -645,7 +694,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -673,7 +722,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -703,7 +752,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -729,7 +778,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -757,7 +806,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -785,7 +834,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -811,7 +860,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -839,7 +888,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -867,7 +916,7 @@ impl AppSink {
             f: glib::ffi::gpointer,
         ) {
             let f: &F = &*(f as *const F);
-            f(&AppSink::from_glib_borrow(this).unsafe_cast_ref())
+            f(AppSink::from_glib_borrow(this).unsafe_cast_ref())
         }
         unsafe {
             let f: Box<F> = Box::new(f);
@@ -988,7 +1037,7 @@ mod tests {
         let videotestsrc = gst::ElementFactory::make("videotestsrc", None).unwrap();
         let appsink = gst::ElementFactory::make("appsink", None).unwrap();
 
-        videotestsrc.set_property("num-buffers", &5).unwrap();
+        videotestsrc.set_property("num-buffers", 5);
 
         let pipeline = gst::Pipeline::new(None);
         pipeline.add(&videotestsrc).unwrap();
