@@ -1,6 +1,9 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 #![cfg_attr(feature = "dox", feature(doc_cfg))]
+#![allow(clippy::missing_safety_doc)]
+#![allow(clippy::non_send_fields_in_send_ty)]
+#![doc = include_str!("../README.md")]
 
 pub use ffi;
 pub use glib;
@@ -15,6 +18,7 @@ macro_rules! skip_assert_initialized {
 #[allow(clippy::too_many_arguments)]
 #[allow(clippy::match_same_arms)]
 #[allow(clippy::use_self)]
+#[allow(unused_imports)]
 mod auto;
 pub use crate::auto::*;
 
@@ -29,6 +33,4 @@ pub use crate::app_src::AppSrcCallbacks;
 pub mod prelude {
     #[doc(hidden)]
     pub use gst_base::prelude::*;
-
-    pub use crate::auto::traits::*;
 }
